from django.contrib.auth.models import User
from django.test import TestCase
from django_dynamic_fixture import G
from mock import Mock

from user_guide import models, views


class GuideSeenTest(TestCase):

    def test_post(self):
        user = G(User)
        guide_info = G(models.GuideInfo, user=user)
        request = Mock(user=user, POST={
            'id': guide_info.id,
            'is_finished': True
        })
        view = views.GuideSeenView()

        self.assertEqual(view.post(request).status_code, 200)
        self.assertTrue(models.GuideInfo.objects.get(id=guide_info.id).is_finished)

    def test_post_wrong_user(self):
        user1 = G(User)
        user2 = G(User)
        guide_info = G(models.GuideInfo, user=user1)
        request = Mock(user=user2, POST={
            'id': guide_info.id,
            'is_finished': True
        })
        view = views.GuideSeenView()

        self.assertEqual(view.post(request).status_code, 200)
        self.assertFalse(models.GuideInfo.objects.get(id=guide_info.id).is_finished)



"""
Template tag for displaying user guides.
"""
import re

from django import template
from django.conf import settings
from django.template import loader
from django.template.defaulttags import CsrfTokenNode

from user_guide.models import GuideInfo


register = template.Library()

# The maximum number of guides to show per page
USER_GUIDE_SHOW_MAX = getattr(settings, 'USER_GUIDE_SHOW_MAX', 10)

# Use cookies to determine if guides should be shown
USER_GUIDE_USE_COOKIES = getattr(settings, 'USER_GUIDE_USE_COOKIES', False)

# The url to any custom CSS
USER_GUIDE_CSS_URL = getattr(
    settings,
    'USER_GUIDE_CSS_URL',
    None
)

# The url to any custom JS
USER_GUIDE_JS_URL = getattr(
    settings,
    'USER_GUIDE_JS_URL',
    None
)


@register.simple_tag(takes_context=True)
def user_guide(context, *args, **kwargs):
    """
    Creates html items for all appropriate user guides.

        Kwargs:
            guide_name: A string name of a specific guide.
            guide_tags: An array of string guide tags.
            limit: An integer maxmimum number of guides to show at a single time.

        Returns:
            An html string containing the user guide scaffolding and any guide html.
    """
    user = context['request'].user if 'request' in context and hasattr(context['request'], 'user') else None

    if user and user.is_authenticated():  # No one is logged in
        limit = kwargs.get('limit', USER_GUIDE_SHOW_MAX)
        filters = {
            'user': user,
            'is_finished': False
        }

        # Handle special filters
        if kwargs.get('guide_name'):
            filters['guide__guide_name'] = kwargs.get('guide_name')
        if kwargs.get('guide_tags'):
            filters['guide__guide_tag__in'] = kwargs.get('guide_tags')

        # Set the html
        html = ''.join((
            '<div data-guide="{0}" class="django-user-guide-item">{1}</div>'.format(
                guide_info.id,
                guide_info.guide.html
            ) for guide_info in GuideInfo.objects.select_related('guide').filter(**filters).only('guide')[:limit]
        ))

        # Return the rendered template with the guide html
        return loader.render_to_string('user_guide/window.html', {
            'html': re.sub(r'\{\s*static\s*\}', settings.STATIC_URL, html),
            'css_href': '{0}user_guide/build/django-user-guide.css'.format(settings.STATIC_URL),
            'js_src': '{0}user_guide/build/django-user-guide.js'.format(settings.STATIC_URL),
            'custom_css_href': USER_GUIDE_CSS_URL,
            'custom_js_src': USER_GUIDE_JS_URL,
            'use_cookies': str(USER_GUIDE_USE_COOKIES).lower(),
            'csrf_node': CsrfTokenNode().render(context)
        })
    else:
        return ''



from django.conf import settings

from user_guide.models import Guide, GuideInfo

if 'django.contrib.admin' in settings.INSTALLED_APPS:
    from django.contrib import admin

    class GuideAdmin(admin.ModelAdmin):
        list_display = ('guide_name', 'guide_tag', 'guide_importance', 'creation_time')

    class GuideInfoAdmin(admin.ModelAdmin):
        list_display = ('user', 'guide_name', 'is_finished', 'finished_time')

        def guide_name(self, obj):
            return obj.guide.guide_name

    admin.site.register(Guide, GuideAdmin)
    admin.site.register(GuideInfo, GuideInfoAdmin)



from django.conf import settings
from django.db import models
import six


@six.python_2_unicode_compatible
class Guide(models.Model):
    """
    Describes a guide to be tied to any number of users.
    """
    # The html that should be rendered in a guide.
    html = models.TextField()
    # The type of guide to render. The only guide type currently supported is 'Window.'
    guide_type = models.CharField(max_length=16, choices=(('WINDOW', 'Window'),), default='WINDOW')
    # The name of the guide. Mainly for display purposes.
    guide_name = models.CharField(max_length=64, unique=True)
    # A tag for the given guide. For filtering purposes.
    guide_tag = models.TextField(default='all')
    # An ordering parameter for the guide. To show a guide first, give it a larger guide_importance.
    guide_importance = models.IntegerField(default=0)
    # The creation time of the guide.
    creation_time = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return str(self.guide_name)


class GuideInfo(models.Model):
    """
    Ties a guide to a user.
    """
    # The user that should see this guide.
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    # The guide that should be shown to the user.
    guide = models.ForeignKey(Guide)
    # Has the guide been seen by a user?
    is_finished = models.BooleanField(default=False)
    # Save the finished time for convenience
    finished_time = models.DateTimeField(null=True, blank=True)

    class Meta:
        unique_together = ('user', 'guide')
        ordering = ['-guide__guide_importance', 'guide__creation_time']



from django.contrib.auth.models import User
from django.http import HttpRequest
from django.template import Context, Template
from django.test import TestCase

from user_guide.models import Guide, GuideInfo


class TemplateTagTest(TestCase):
    def setUp(self):
        super(TemplateTagTest, self).setUp()

        self.users = [
            User.objects.create_user(
                username='test{0}@test.com'.format(i),
                email='test{0}@test.com'.format(i),
                password='test'
            )
            for i in range(0, 5)
        ]

        self.guides = [
            Guide.objects.create(
                html='<div>Hello test {0}! <img src="{{static}}images/cool-image.png"></div>'.format(i),
                guide_name='Test Guide {0}'.format(i),
                guide_type='Window',
                guide_tag='tag{0}'.format(i),
                guide_importance=i
            )
            for i in range(0, 10)
        ]

    def test_user_guide_tags_no_user(self):
        t = Template('{% load user_guide_tags %}{% user_guide %}')
        c = Context({})
        self.assertEqual('', t.render(c))

    def test_user_guide_tags_no_filters(self):
        t = Template('{% load user_guide_tags %}{% user_guide %}')
        r = HttpRequest()
        r.user = self.users[0]
        c = Context({
            'request': r,
            'csrf_token': '1234'
        })

        # Create an info for each guide
        guide_infos = [GuideInfo.objects.create(user=self.users[0], guide=guide) for guide in self.guides]

        # Render the template
        rendered = t.render(c)

        # Make sure the correct guides show up, guide_order should apply here
        self.assertTrue('<input type=\'hidden\' name=\'csrfmiddlewaretoken\' value=\'1234\' />' in rendered)
        self.assertTrue('Hello test 9!' in rendered)
        self.assertTrue('data-guide="{0}"'.format(guide_infos[9].id) in rendered)
        self.assertTrue('Hello test 8!' in rendered)
        self.assertTrue('data-guide="{0}"'.format(guide_infos[8].id) in rendered)
        self.assertTrue('Hello test 7!' in rendered)
        self.assertTrue('data-guide="{0}"'.format(guide_infos[7].id) in rendered)
        self.assertTrue('Hello test 6!' in rendered)
        self.assertTrue('data-guide="{0}"'.format(guide_infos[6].id) in rendered)
        self.assertTrue('Hello test 5!' in rendered)
        self.assertTrue('data-guide="{0}"'.format(guide_infos[5].id) in rendered)
        self.assertTrue('Hello test 4!' not in rendered)  # Should not have rendered 6 guides
        self.assertTrue('Hello test 3!' not in rendered)  # Should not have rendered 7 guides
        self.assertTrue('Hello test 2!' not in rendered)  # Should not have rendered 8 guides
        self.assertTrue('Hello test 1!' not in rendered)  # Should not have rendered 9 guides
        self.assertTrue('Hello test 0!' not in rendered)  # Should not have rendered 10 guides
        self.assertTrue('django-user-guide.css' in rendered)  # Should have django-user-guide style sheet
        self.assertTrue('django-user-guide.js' in rendered)  # Should have django-user-guide script
        self.assertTrue('custom-style.css' in rendered)  # Should have custom style sheet
        self.assertTrue('custom-script.js' in rendered)  # Should have custom script
        self.assertTrue('<img src="/static/images/cool-image.png">' in rendered)  # Should have expanded {static}

    def test_user_guide_tags_guide_name_filter(self):
        t = Template('{% load user_guide_tags %}{% user_guide guide_name=guide_name %}')
        r = HttpRequest()
        r.user = self.users[0]
        c = Context({
            'request': r,
            'guide_name': 'Test Guide 1'
        })

        # create a guide info
        GuideInfo.objects.create(user=self.users[0], guide=self.guides[1])

        # render the template
        rendered = t.render(c)

        self.assertTrue('<div>Hello test 1!' in rendered)

    def test_user_guide_tags_guide_tag_filter(self):
        t = Template('{% load user_guide_tags %}{% user_guide guide_tags=guide_tags %}')
        r = HttpRequest()
        r.user = self.users[0]
        c = Context({
            'request': r,
            'guide_tags': ['tag0', 'tag1'],
            'csrf_token': '1234'
        })

        # create a few guide infos
        GuideInfo.objects.create(user=self.users[0], guide=self.guides[0])
        GuideInfo.objects.create(user=self.users[0], guide=self.guides[1])

        # render the template
        rendered = t.render(c)

        self.assertTrue('<input type=\'hidden\' name=\'csrfmiddlewaretoken\' value=\'1234\' />' in rendered)
        self.assertTrue('<div>Hello test 0!' in rendered)
        self.assertTrue('<div>Hello test 1!' in rendered)



from django.test import TestCase

from user_guide import urls


class UrlsTest(TestCase):
    def test_that_urls_are_defined(self):
        """
        Should have several urls defined.
        """
        self.assertEqual(len(urls.urlpatterns), 1)
        self.assertEqual(urls.urlpatterns[0].name, 'user_guide.seen')



import six
if six.PY3:  # pragma: no cover
    from importlib import reload

from django.conf import settings
from django.test import TestCase

from user_guide import admin


class NoAdminTest(TestCase):
    """
    Tests loading of the admin module when django.contrib.admin is not installed.
    """
    def test_no_admin(self):
        with self.settings(INSTALLED_APPS=[app for app in settings.INSTALLED_APPS if app != 'django.contrib.admin']):
            reload(admin)
            self.assertIsNotNone(admin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Guide',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, verbose_name='ID', serialize=False)),
                ('html', models.TextField()),
                ('guide_type', models.CharField(max_length=16, default='WINDOW', choices=[('WINDOW', 'Window')])),
                ('guide_name', models.CharField(unique=True, max_length=64)),
                ('guide_tag', models.TextField(default='all')),
                ('guide_importance', models.IntegerField(default=0)),
                ('creation_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GuideInfo',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, verbose_name='ID', serialize=False)),
                ('is_finished', models.BooleanField(default=False)),
                ('finished_time', models.DateTimeField(null=True, blank=True)),
                ('guide', models.ForeignKey(to='user_guide.Guide')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-guide__guide_importance', 'guide__creation_time'],
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='guideinfo',
            unique_together=set([('user', 'guide')]),
        ),
    ]



from django.http import HttpResponse
from django.views.generic import View

from user_guide import models


class GuideSeenView(View):
    def post(self, request):
        guide_id = request.POST.get('id', 0)
        is_finished = request.POST.get('is_finished', False)
        guide_info = models.GuideInfo.objects.get(id=guide_id)

        if guide_info and guide_info.user.id == request.user.id and is_finished:
            guide_info.is_finished = True
            guide_info.save()

        return HttpResponse(status=200)



from django.test import TestCase
from django_dynamic_fixture import G

from user_guide.models import Guide


class GuideTest(TestCase):

    def test_guide_unicode(self):
        guide_obj = G(Guide, guide_name='test_name')
        self.assertEqual(str(guide_obj), 'test_name')



from django.contrib.admin.sites import AdminSite
from django.test import TestCase
from django_dynamic_fixture import G, F

from user_guide.admin import GuideAdmin, GuideInfoAdmin
from user_guide.models import Guide, GuideInfo


class AdminTest(TestCase):
    def setUp(self):
        super(AdminTest, self).setUp()
        self.site = AdminSite()

    def test_user_guide_admin(self):
        guide_admin = GuideAdmin(Guide, self.site)
        self.assertEqual(guide_admin.list_display, ('guide_name', 'guide_tag', 'guide_importance', 'creation_time'))

    def test_user_guide_info_admin(self):
        guide_info_admin = GuideInfoAdmin(GuideInfo, self.site)
        guide_info_obj = G(GuideInfo, guide=F(guide_name='test_name'))
        self.assertEqual(guide_info_admin.list_display, ('user', 'guide_name', 'is_finished', 'finished_time'))
        self.assertEqual(guide_info_admin.guide_name(guide_info_obj), 'test_name')
